@@ -69,6 +69,8 @@ extern int ex(unsigned char* str, unsigned char* pattern,
   else { /* error */
     char s[ONIG_MAX_ERROR_MESSAGE_LEN];
     onig_error_code_to_str((UChar* )s, r);
+    onig_region_free(region, 1 /* 1:free self, 0:free contents only */);
+    onig_free(reg);
     return -1;
   }
 
